//  ANLEGEN DER FUNKTION findGlas

/*Diese Funktion sorgt zunächst dafür, dass der Drehteller mit dem Ultraschallsensor an die
   untere Grenze der gerade spielenden Person gedreht wird. Da der Sensor einen kegelförmigen
   Sensorbereich besitzt, wird ausgeschlossen, dass der Bereich des Nebenspielenden abgetastet
   wird, indem der Scan erst ein paar grad später beginnt und früher endet. Es gibt somkt einen
   kleinen Bereich, nah am Roboter und nah am Rand, der nicht erkannt werden kann.
   Beim Abtasten des vorgegebenen Bereichs erknnt der Sensor den am nächsten am Spiel stehenden
   Gegenstand als Glas und speichert die Position und die Entfernung.
   Anschließend dreht sich der Drehteller auf diese Position, und die Getränkeausgabe kann beginnen.
   Dabei wird der globale Schrittzähler mitgezählt, um die absolute Position weiter zu erfassen.
*/

// Anschlüsse am Arduino:
// GND an GND
// Echo an digital PIN 13
// Trig an digital PIN 2
// VCC an 5V

void findGlas() {

  pinMode(triggerPin, OUTPUT);
  pinMode(echoPin, INPUT);
  pinMode(directionPin1, OUTPUT);
  pinMode(stepPin1, OUTPUT);


  // Zusätzliche benötigte Variablen

  long distance; // Abstand zum Messobjekt
  long duration; // Dauer zwischen Schallaussendung und Echorücklauf
  int counter = 0; // Zählt die Anzahl der Versuche, ein Glas zu finden
  int stepsAfterGlas = 0; // Zählt die zurückgelegten Motorschritte nach dem Glas, um später dorthin zurück zu gehen

  // In der Variable glasDistance wird der Abstand des Glasen abgespeichert.
  // Zu Beginn muss der Wert größer sein, als der maximal erlaubte Abstand, um später einen Vergleich anstellen zu können.

  float glasDistance = maximumRange + 1;
  int glasPosition = 0;

  // Anlegen von Variablen für den Start und das Ende der Messung, measuringDistance ist die Differenz in Motorschritten
  // Mit Hilfe der Modulo Funktion wird ermöglicht, Differenze richtig zu berechnen, auch wenn die 0° Marke überschritten wird

  int stepsPerRotationOfPlate = 360 * degreeInSteps;
  int startStep = (ppA[previousPlayer][1] + 15) % 360 * degreeInSteps;
  int endStep = (ppA[previousPlayer][3] - 15) % 360 * degreeInSteps;
  int measuringDistance = (endStep - startStep + stepsPerRotationOfPlate) % stepsPerRotationOfPlate;

  Serial.print("Die Messung startet bei Schritt ");   // nur für den Entwicklungsmodus
  Serial.print(startStep);                            // nur für den Entwicklungsmodus
  Serial.println(".");                                // nur für den Entwicklungsmodus
  Serial.print("Die Messung endet bei Schritt ");     // nur für den Entwicklungsmodus
  Serial.print(endStep);                              // nur für den Entwicklungsmodus
  Serial.println(".");                                // nur für den Entwicklungsmodus
  Serial.print("Die Länge des Messbereichs beträgt ");// nur für den Entwicklungsmodus
  Serial.print(measuringDistance);                    // nur für den Entwicklungsmodus
  Serial.println(" Schritte.");                       // nur für den Entwicklungsmodus
  Serial.println("-----------------------------------");// Nur für den Entwicklungsmodus

  // Drehe den Motor zur Starposition der Messung
  // Motorrotation gegen den Uhrzeigersinn. Da die aktuelle Position genau in der Mitte des Messbereichs liegt,
  // muss der Teller genau die Hälfte des Messbereichs zurückgedreht werden.

  for (int i = 0; i < 0.5 * measuringDistance; i++) {

    digitalWrite(directionPin1, LOW);

    digitalWrite(stepPin1, HIGH);
    delayMicroseconds(stepperSpeed);
    digitalWrite(stepPin1, LOW);
    delayMicroseconds(stepperSpeed);

    stepCounter--;

    //  Serial.print("Der aktuelle stepCounter ist: ");       // nur für den Entwicklungsmodus
    //  Serial.println(stepCounter);                            // nur für den Entwicklungsmodus
    //  Serial.println("-----------------------------------");// Nur für den Entwicklungsmodus
  }

  for (counter = 0; counter < 3; counter++) {
    // Schritt für Schritt wird der zu messende Bereich abgetastet. Nach jeder Messung wird der Motor einen Schritt weiter gestellt.

    for (int i = 0; i < measuringDistance; i++) {

      // Start der Abstandsmessung mit einer Signallänge von 10 Mikrosekunden
      digitalWrite(triggerPin, HIGH);
      delayMicroseconds(10);
      digitalWrite(triggerPin, LOW);

      // Warten auf das Echo und messen, wie lang das Signal ist
      duration = pulseIn(echoPin, HIGH);

      // Umrechnung der Zeit in Entfernug
      distance = duration / 58.2;

      // Prüfen, ob ein Gegenstand im vorgegeben Bereich / min. max. Abstand steht
      // Wenn kein Gals im erreichbaren Bereich steht, mache nichts und gehe zum nächsten Schritt:
      if (distance > maximumRange || distance < minimumRange) {
      }

      // Wenn ein Gals im erreichbaren Bereich steht, vergleiche den aktuell gemessenen Abstand mit dem zuvor gespeichertne Abstand
      // Wenn der Abstand kleiner ist, als der zuvor gemessene, überschreibe den alten Wert mit dem neuen Abstand und speichere die Position:

      else {
        if ( distance < glasDistance) {
          glasDistance  = distance;
          glasPosition = startStep + i;
          stepsAfterGlas = 0;
        }
      }

      // Dann wird der Motor einen Schritt weitergedreht
      // Motorrotation im Uhrzeigersinn

      digitalWrite(directionPin1, HIGH);

      digitalWrite(stepPin1, HIGH);
      delayMicroseconds(stepperSpeed);
      digitalWrite(stepPin1, LOW);
      delayMicroseconds(stepperSpeed);

      stepCounter++;

      stepsAfterGlas++;


      // Printing the calculated distance                     // nur für den Entwicklungsmodus
      // Serial.print("Die Distanz zum Glas betraegt:");         // nur für den Entwicklungsmodus
      //  Serial.print(glasDistance);                             // nur für den Entwicklungsmodus
      // Serial.println("cm");                                   // nur für den Entwicklungsmodus
      // Serial.print("Der aktuelle stepCounter ist: ");         // nur für den Entwicklungsmodus
      // Serial.println(stepCounter);                            // nur für den Entwicklungsmodus
      //  Serial.println("-----------------------------------");  // nur für den Entwicklungsmodus




      // Pause nach jedem Schritt
      delay(5);                                             // nur für den Entwicklungsmodus, um Text lesen zu können. Später viel schneller
    }

    // Wenn kein Glas im erreichbaren Bereich gefunden wurde, gib einen Hinweis aus und warte anschließend 5 Sekunden
    // Nach 3 Versuchen wird die Suche abgebrochen

    if (glasDistance == maximumRange + 1) {
      if (counter < 2) {
        Serial.println("Kein Glas gefunden. Bitte stelle das Glas in den erreichbaren Bereich."); // nur für den Entwicklungsmodus, wäre aber auch gut als Diplay, vielleicht noch einbauen
        Serial.println("-----------------------------------");

        // Drehe den Motor zur Starposition der Messung
        // Motorrotation gegen den Uhrzeigersinn
        // Da die Aktuelle Position das obere Ende des Messbereichs ist, ist der Weg zum Zurückdrehen genau ein mal der Messbereich.

        Serial.println("Der Motor wird zur Startposition der Messung gedreht.");         // nur für den Entwicklungsmodus

        for (int i = 0; i < measuringDistance; i++) {

          digitalWrite(directionPin1, LOW);

          digitalWrite(stepPin1, HIGH);
          delayMicroseconds(stepperSpeed);
          digitalWrite(stepPin1, LOW);
          delayMicroseconds(stepperSpeed);

          stepCounter--;

          //   Serial.print("Der aktuelle stepCounter ist: ");         // nur für den Entwicklungsmodus
          //   Serial.println(stepCounter);                            // nur für den Entwicklungsmodus
          //   Serial.println("-----------------------------------");  // nur für den Entwicklungsmodus
        }
      }
      else {
        Serial.println("Kein Glas gefunden. Die Suche wird abgebrochen.");                        // nur für den Entwicklungsmodus, wäre aber auch gut als Diplay, vielleicht noch einbauen
        Serial.println("-----------------------------------");                                    // nur für den Entwicklungsmodus, wäre aber auch gut als Diplay, vielleicht noch einbauen

        // Drehe den Motor zurück auf die Position dre/des Spielenden
        // Motorrotation gegen den Uhrzeigersinn. Da die aktuelle Position genau am Ende des Messbereichs liegt,
        // muss der Teller genau die Hälfte des Messbereichs zurückgedreht werden.

        Serial.println("Der Motor wird zur/zum Spielenden gedreht.");         // nur für den Entwicklungsmodus

        for (int i = 0; i < 0.5 * measuringDistance; i++) {

          digitalWrite(directionPin1, LOW);

          digitalWrite(stepPin1, HIGH);
          delayMicroseconds(stepperSpeed);
          digitalWrite(stepPin1, LOW);
          delayMicroseconds(stepperSpeed);

          stepCounter--;

          //    Serial.print("Der aktuelle stepCounter ist: ");         // nur für den Entwicklungsmodus
          //    Serial.println(stepCounter);                            // nur für den Entwicklungsmodus
          //    Serial.println("-----------------------------------");  // nur für den Entwicklungsmodus
        }

        // Hier kommt dann die Funktion nextPlayer, es wird kein Getränk ausgegeben.
      }
    }

    else {
      counter = 3;
      Serial.print("Das Glas steht auf ");                                                        // nur für den Entwicklungsmodus, wäre aber auch gut als Diplay, vielleicht noch einbauen
      Serial.print(glasPosition * stepsInDegree);                                                 // nur für den Entwicklungsmodus, wäre aber auch gut als Diplay, vielleicht noch einbauen
      Serial.println(" Grad.");                                                                   // nur für den Entwicklungsmodus, wäre aber auch gut als Diplay, vielleicht noch einbauen
      Serial.print("Die Entfernung beträgt ");                                                   // nur für den Entwicklungsmodus, wäre aber auch gut als Diplay, vielleicht noch einbauen
      Serial.print(glasDistance);                                                                 // nur für den Entwicklungsmodus, wäre aber auch gut als Diplay, vielleicht noch einbauen
      Serial.println(" cm.");
      Serial.println(stepsAfterGlas);  // nur für den Entwicklungsmodus, wäre aber auch gut als Diplay, vielleicht noch einbauen
      Serial.println("-----------------------------------");                                      // nur für den Entwicklungsmodus, wäre aber auch gut als Diplay, vielleicht noch einbauen

      // Dann wird der Motor so viele Schritte zurück gedreht, wie seit dem Glas vergangen sind

      Serial.println("Der Motor wird zum Glas gedreht.");         // nur für den Entwicklungsmodus

      for (int i = 0; i < stepsAfterGlas; i++) {

        // Motorrotation gegen den Uhrzeigersinn
        digitalWrite(directionPin1, LOW);

        digitalWrite(stepPin1, HIGH);
        delayMicroseconds(stepperSpeed);
        digitalWrite(stepPin1, LOW);
        delayMicroseconds(stepperSpeed);

        stepCounter--;

        //    Serial.print("Der aktuelle stepCounter ist: ");         // nur für den Entwicklungsmodus
        //    Serial.println(stepCounter);                            // nur für den Entwicklungsmodus
        //    Serial.println("-----------------------------------");  // nur für den Entwicklungsmodus
      }

      // Warte eine Sekunde, bevor die Getränkeausgabe beginnt

      delay(1000);
      Serial.println("Eine Sekunde warten vor der Getränkeausgabe");            // nur für den Entwicklungsmodus

      // Hier startet dann die Getränkeausgabe
      serveDrinkNew();



      // Nach der Getränkeausgabe fährt der Drehteller zunächst wieder zurück an das Obere Ende des
      // Messbereicht, um sich anschließend wieder zur Position der/des Spielenden zu drehen.

      Serial.println("Der Motor wird zum oberen Ende des Messbereichs gedreht.");         // nur für den Entwicklungsmodus

      for (int i = 0; i < stepsAfterGlas; i++) {

        // Motorrotation im Uhrzeigersinn
        digitalWrite(directionPin1, HIGH);

        digitalWrite(stepPin1, HIGH);
        delayMicroseconds(stepperSpeed);
        digitalWrite(stepPin1, LOW);
        delayMicroseconds(stepperSpeed);

        stepCounter++;

        //    Serial.print("Der aktuelle stepCounter ist: ");         // nur für den Entwicklungsmodus
        //    Serial.println(stepCounter);                            // nur für den Entwicklungsmodus
        //    Serial.println("-----------------------------------");  // nur für den Entwicklungsmodus
      }
      delay(1000); // Warte eine Sekunde vor dem Richtungswechsel

      Serial.println("Der Motor wird zur/zum Spielenden gedreht.");         // nur für den Entwicklungsmodus

      for (int i = 0; i < 0.5 * measuringDistance; i++) {

        // Motorrotation gegen den Uhrzeigersinn
        digitalWrite(directionPin1, LOW);

        digitalWrite(stepPin1, HIGH);
        delayMicroseconds(stepperSpeed);
        digitalWrite(stepPin1, LOW);
        delayMicroseconds(stepperSpeed);

        stepCounter--;

        //    Serial.print("Der aktuelle stepCounter ist: ");         // nur für den Entwicklungsmodus
        //    Serial.println(stepCounter);                            // nur für den Entwicklungsmodus
        //    Serial.println("-----------------------------------");  // nur für den Entwicklungsmodus
      }
      // hier kommt dann die Funktion nextPlayer
    }
    delay(5000);
  }
}
